package com.jachdev.consumerprotection.ui.alarm;

import android.app.AlarmManager;
import android.app.DatePickerDialog;
import android.app.PendingIntent;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.widget.DatePicker;
import android.widget.TimePicker;

import com.jachdev.commonlibs.base.BaseActivity;
import com.jachdev.commonlibs.utils.DateTimeUtil;
import com.jachdev.commonlibs.widget.CustomButton;
import com.jachdev.commonlibs.widget.CustomTextView;
import com.jachdev.consumerprotection.R;
import com.jachdev.consumerprotection.data.User;
import com.jachdev.consumerprotection.receiver.AlarmReceiver;
import com.jachdev.consumerprotection.util.SessionManager;

import java.util.Calendar;

import butterknife.BindView;
import butterknife.OnClick;

public class AddAlarmActivity extends BaseActivity {

    private static final String TAG = AddAlarmActivity.class.getSimpleName();
    @BindView(R.id.tv_name)
    CustomTextView tvName;
    @BindView(R.id.tv_age)
    CustomTextView tvAge;

    @BindView(R.id.btnBedInTime)
    CustomButton btnBedInTime;
    @BindView(R.id.btnFallenSleep)
    CustomButton btnFallenSleep;
    @BindView(R.id.btnRecommendTime)
    CustomButton btnRecommendTime;
    @BindView(R.id.btnRingingTime)
    CustomButton btnRingingTime;

    private AlarmManager alarmMgr;
    private PendingIntent alarmIntent;
    private Calendar selectedCalendar;

    @Override
    protected int layoutRes() {
        return R.layout.activity_add_alarm;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @OnClick(R.id.btnBedInTime)
    public void onBedInTimeClick(){
        showDatePicker(btnBedInTime);
    }

    @OnClick(R.id.btnFallenSleep)
    public void onFallenSleepClick(){
        showDatePicker(btnFallenSleep);
    }

    @OnClick(R.id.btnRecommendTime)
    public void onRecommendTimeClick(){
        showDatePicker(btnRecommendTime);
    }

    @OnClick(R.id.btnRingingTime)
    public void onRingingTimeClick(){
        showDatePicker(btnRingingTime);
    }

    @OnClick(R.id.btnSubmit)
    public void onSubmitClick(){

        alarmMgr = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        Intent intent = new Intent(AddAlarmActivity.this, AlarmReceiver.class);
        alarmIntent = PendingIntent.getBroadcast(AddAlarmActivity.this, 0, intent, 0);

        Calendar calendar = selectedCalendar;

        alarmMgr.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), alarmIntent);

        showMessage("Alarm set successfully");

        AddAlarmActivity.this.finish();
    }

    private void init() {

        User user = SessionManager.getInstance().getUser();

        tvName.setAnyText("Hi " + user.getName());
        tvAge.setAnyText("Age: " + user.getAge() + "");

        Calendar c = Calendar.getInstance();

        btnBedInTime.setText(DateTimeUtil.unixToDateTime(c.getTimeInMillis(), DateTimeUtil.DATE_FORMAT_2));
        btnFallenSleep.setText(DateTimeUtil.unixToDateTime(c.getTimeInMillis(), DateTimeUtil.DATE_FORMAT_2));
        btnRecommendTime.setText(DateTimeUtil.unixToDateTime(c.getTimeInMillis(), DateTimeUtil.DATE_FORMAT_2));
        btnRingingTime.setText(DateTimeUtil.unixToDateTime(c.getTimeInMillis(), DateTimeUtil.DATE_FORMAT_2));
    }

    private void showDatePicker(CustomButton button) {
        Calendar c = Calendar.getInstance();
        Calendar selected = Calendar.getInstance();

        DatePickerDialog datePickerDialog = new DatePickerDialog(this,
                new DatePickerDialog.OnDateSetListener() {

                    @Override
                    public void onDateSet(DatePicker view, int year,
                                          int monthOfYear, int dayOfMonth) {

                        selected.set(year, monthOfYear, dayOfMonth);

                        showTimerPicker(button, selected);
                    }
                }, c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH));
        datePickerDialog.show();
    }

    private void showTimerPicker(CustomButton button, Calendar selected) {
        Calendar c = Calendar.getInstance();
        TimePickerDialog timePickerDialog = new TimePickerDialog(this,
                new TimePickerDialog.OnTimeSetListener() {

                    @Override
                    public void onTimeSet(TimePicker view, int hourOfDay,
                                          int minute) {

                        selected.set(Calendar.HOUR_OF_DAY, hourOfDay);
                        selected.set(Calendar.MINUTE, minute);

                        if(button == btnRingingTime){
                            selectedCalendar = selected;

                            Log.d(TAG, "onDateSet: " + selectedCalendar.getTimeInMillis());
                        }

                        button.setText(DateTimeUtil.unixToDateTime(selected.getTimeInMillis(), DateTimeUtil.DATE_FORMAT_2));
                    }
                }, c.get(Calendar.HOUR_OF_DAY), c.get(Calendar.MINUTE), false);
        timePickerDialog.show();
    }
}