package com.jachdev.consumerprotection.ui.alarm;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import butterknife.BindView;
import butterknife.OnClick;

import android.app.AlarmManager;
import android.content.Intent;
import android.os.Bundle;

import com.github.mikephil.charting.charts.BarChart;
import com.github.mikephil.charting.components.Description;
import com.github.mikephil.charting.components.Legend;
import com.github.mikephil.charting.components.LimitLine;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.components.YAxis;
import com.github.mikephil.charting.data.BarData;
import com.github.mikephil.charting.data.BarDataSet;
import com.github.mikephil.charting.data.BarEntry;
import com.github.mikephil.charting.formatter.IndexAxisValueFormatter;
import com.jachdev.commonlibs.base.BaseActivity;
import com.jachdev.commonlibs.widget.CustomTextView;
import com.jachdev.consumerprotection.R;
import com.jachdev.consumerprotection.data.LipidProfile;
import com.jachdev.consumerprotection.data.SleepData;
import com.jachdev.consumerprotection.ui.home.HomeFragment;
import com.jachdev.consumerprotection.ui.home.HomeViewModel;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class AlarmActivity extends BaseActivity {

    private AlarmViewModel alarmViewModel;
    private List<SleepData> lipidProfiles;

    @BindView(R.id.chart)
    BarChart chart;
    @BindView(R.id.tvXAxisLabel)
    CustomTextView tvXAxisLabel;
    @BindView(R.id.tvYAxisLabel)
    CustomTextView tvYAxisLabel;

    @Override
    protected int layoutRes() {
        return R.layout.activity_alarm;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        alarmViewModel =
                new ViewModelProvider(this).get(AlarmViewModel.class);

        alarmViewModel.getLipidProfileData().observe(AlarmActivity.this, new Observer<List<SleepData>>() {
            @Override
            public void onChanged(List<SleepData> lipidProfiles) {
                if(lipidProfiles == null)
                    return;

                AlarmActivity.this.lipidProfiles = lipidProfiles;
                drawDiagram();
            }
        });
        setXYAxis();
        alarmViewModel.getLipidProfile();

    }

    @OnClick(R.id.btnAlarm)
    public void onClickAddAlarm(){

        activityToActivity(AddAlarmActivity.class);

    }

    @OnClick(R.id.btnStressLevel)
    public void onClickStressLevel(){

        Intent intent = new Intent(AlarmActivity.this, SleepingProgressActivity.class);

        Bundle bundle = new Bundle();
        bundle.putInt(SleepingProgressActivity.KEY_TITLE, R.string.stress_level);
        intent.putExtras(bundle);

        startActivity(intent);

    }

    private void setXYAxis() {
        chart.clear();
        chart.getAxisLeft().removeAllLimitLines();

        tvXAxisLabel.setAnyText("Time");

        String yLabel = "Duration";
        tvYAxisLabel.setAnyText(yLabel);

        //hiding the grey background of the chart, default false if not set
        chart.setDrawGridBackground(false);
        //remove the bar shadow, default false if not set
        chart.setDrawBarShadow(false);
        //remove border of the chart, default false if not set
        chart.setDrawBorders(false);

        //remove the description label text located at the lower right corner
        Description description = new Description();
        description.setEnabled(false);
        chart.setDescription(description);

        //setting animation for y-axis, the bar will pop up from 0 to its value within the time we set
        chart.animateY(500);
        //setting animation for x-axis, the bar will pop up separately within the time we set
        chart.animateX(500);

        XAxis xAxis = chart.getXAxis();
        //change the position of x-axis to the bottom
        xAxis.setPosition(XAxis.XAxisPosition.BOTTOM);
        //set the horizontal distance of the grid line
        xAxis.setGranularity(1f);
        //hiding the x-axis line, default true if not set
        xAxis.setDrawAxisLine(false);
        //hiding the vertical grid lines, default true if not set
        xAxis.setDrawGridLines(false);

        YAxis leftAxis = chart.getAxisLeft();
        //hiding the left y-axis line, default true if not set
        leftAxis.setDrawAxisLine(true);

        YAxis rightAxis = chart.getAxisRight();
        //hiding the right y-axis line, default true if not set
        rightAxis.setDrawAxisLine(false);

        Legend legend = chart.getLegend();
        //setting the shape of the legend form to line, default square shape
        legend.setForm(Legend.LegendForm.LINE);
        //setting the text size of the legend
        legend.setTextSize(8f);
        //setting the alignment of legend toward the chart
        legend.setVerticalAlignment(Legend.LegendVerticalAlignment.BOTTOM);
        legend.setHorizontalAlignment(Legend.LegendHorizontalAlignment.LEFT);
        //setting the stacking direction of legend
        legend.setOrientation(Legend.LegendOrientation.HORIZONTAL);
        //setting the location of legend outside the chart, default false if not set
        legend.setDrawInside(false);

        String limitLineName = "";

        int maxCapacity = 60 * 60 * 60 * 10;
        LimitLine ll = new LimitLine(maxCapacity, "Max " + limitLineName);
        chart.getAxisLeft().addLimitLine(ll);

        int minCapacity = 60 * 60 * 60 * 8;
        LimitLine l2 = new LimitLine(minCapacity, "Min " + limitLineName);
        chart.getAxisLeft().addLimitLine(l2);
    }

    private void drawDiagram() {
        List<BarEntry> entries = new ArrayList<BarEntry>();
        List<Integer> colors = new ArrayList<Integer>();
        int max = 0;
        int min = 1000000;

        for (int x = 0; x < lipidProfiles.size(); x++) {
            SleepData d = lipidProfiles.get(x);
            long date = d.getCreated_at();
            Calendar calendar = Calendar.getInstance();
            calendar.setTimeInMillis(date*1000);
            int value = 0;

            value = d.getDuration();

            if(max < value)
                max = value;

            if(min > value)
                min = value;

            int maxCapacity = 60 * 60 * 60 * 10;
            int minCapacity = 60 * 60 * 60 * 8;

            if(value >= maxCapacity || value <= minCapacity){
                colors.add(ActivityCompat.getColor(AlarmActivity.this, R.color.color_danger));
            }else if(isMiddleOf(value, maxCapacity) || isMiddleOf(value, minCapacity)){
                colors.add(ActivityCompat.getColor(AlarmActivity.this, R.color.color_middle));
            }else{
                colors.add(ActivityCompat.getColor(AlarmActivity.this, R.color.color_normal));
            }

            if(date > 0 && value > 0){

                BarEntry entry = new BarEntry(calendar.get(Calendar.MONTH), value);
                entries.add(entry);
            }
        }

        BarDataSet barDataSet = new BarDataSet(entries, "Duration");

        barDataSet.setColors(colors);
        //Setting the size of the form in the legend
        barDataSet.setFormSize(5f);
        //showing the value of the bar, default true if not set
        barDataSet.setDrawValues(true);
        //setting the text size of the value of the bar
        barDataSet.setValueTextSize(12f);

        BarData data = new BarData(barDataSet);
        chart.setData(data);

        YAxis yAxis = chart.getAxisLeft();
        yAxis.setAxisMinimum(0);
        yAxis.setAxisMaximum(max+10f);

        XAxis xAxis = chart.getXAxis();
        xAxis.setValueFormatter(new StickyDateAxisValueFormatter());

        chart.invalidate();
    }

    private boolean isMiddleOf(int value, int level) {
        int min = level - 5;
        int max = level + 5;

        return value > min && value < max;
    }

    public class StickyDateAxisValueFormatter extends IndexAxisValueFormatter {

        @Override
        public String getFormattedValue(float value) {

            // Convert float value to date string
            // Convert from days back to milliseconds to format time  to show to the user
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.MONTH, (int) value);
            long emissionsMilliSince1970Time = (long) value*1000;
            // Show time in local version
            Date timeMilliseconds = calendar.getTime();
            SimpleDateFormat dateTimeFormat = new SimpleDateFormat("MMM");

            return dateTimeFormat.format(timeMilliseconds);
        }
    }

}